@@ -8,19 +8,24 @@ package connections
 
 import (
 	"bytes"
+	"context"
 	crand "crypto/rand"
 	"io"
 	"math/rand"
+	"sync/atomic"
 	"testing"
 
+	"golang.org/x/time/rate"
+
 	"github.com/syncthing/syncthing/lib/config"
 	"github.com/syncthing/syncthing/lib/events"
 	"github.com/syncthing/syncthing/lib/protocol"
-	"golang.org/x/time/rate"
 )
 
-var device1, device2, device3, device4 protocol.DeviceID
-var dev1Conf, dev2Conf, dev3Conf, dev4Conf config.DeviceConfiguration
+var (
+	device1, device2, device3, device4     protocol.DeviceID
+	dev1Conf, dev2Conf, dev3Conf, dev4Conf config.DeviceConfiguration
+)
 
 func init() {
 	device1, _ = protocol.DeviceIDFromString("AIR6LPZ7K4PTTUXQSMUUCPQ5YWOEDFIIQJUG7772YQXXR5YD6AWQ")
@@ -29,24 +34,37 @@ func init() {
 	device4, _ = protocol.DeviceIDFromString("P56IOI7-MZJNU2Y-IQGDREY-DM2MGTI-MGL3BXN-PQ6W5BM-TBBZ4TJ-XZWICQ2")
 }
 
-func initConfig() config.Wrapper {
-	cfg := config.Wrap("/dev/null", config.New(device1), device1, events.NoopLogger)
-	dev1Conf = config.NewDeviceConfiguration(device1, "device1")
-	dev2Conf = config.NewDeviceConfiguration(device2, "device2")
-	dev3Conf = config.NewDeviceConfiguration(device3, "device3")
-	dev4Conf = config.NewDeviceConfiguration(device4, "device4")
+func newDeviceConfiguration(w config.Wrapper, id protocol.DeviceID, name string) config.DeviceConfiguration {
+	cfg := w.DefaultDevice()
+	cfg.DeviceID = id
+	cfg.Name = name
+	return cfg
+}
+
+func initConfig() (config.Wrapper, context.CancelFunc) {
+	wrapper := config.Wrap("/dev/null", config.New(device1), device1, events.NoopLogger)
+	dev1Conf = newDeviceConfiguration(wrapper, device1, "device1")
+	dev2Conf = newDeviceConfiguration(wrapper, device2, "device2")
+	dev3Conf = newDeviceConfiguration(wrapper, device3, "device3")
+	dev4Conf = newDeviceConfiguration(wrapper, device4, "device4")
+
+	ctx, cancel := context.WithCancel(context.Background())
+	go wrapper.Serve(ctx)
 
 	dev2Conf.MaxRecvKbps = rand.Int() % 100000
 	dev2Conf.MaxSendKbps = rand.Int() % 100000
 
-	waiter, _ := cfg.SetDevices([]config.DeviceConfiguration{dev1Conf, dev2Conf, dev3Conf, dev4Conf})
+	waiter, _ := wrapper.Modify(func(cfg *config.Configuration) {
+		cfg.SetDevices([]config.DeviceConfiguration{dev1Conf, dev2Conf, dev3Conf, dev4Conf})
+	})
 	waiter.Wait()
-	return cfg
+	return wrapper, cancel
 }
 
 func TestLimiterInit(t *testing.T) {
-	cfg := initConfig()
-	lim := newLimiter(device1, cfg)
+	wrapper, wrapperCancel := initConfig()
+	defer wrapperCancel()
+	lim := newLimiter(device1, wrapper)
 
 	device2ReadLimit := dev2Conf.MaxRecvKbps
 	device2WriteLimit := dev2Conf.MaxSendKbps
@@ -70,8 +88,9 @@ func TestLimiterInit(t *testing.T) {
 }
 
 func TestSetDeviceLimits(t *testing.T) {
-	cfg := initConfig()
-	lim := newLimiter(device1, cfg)
+	wrapper, wrapperCancel := initConfig()
+	defer wrapperCancel()
+	lim := newLimiter(device1, wrapper)
 
 	// should still be inf/inf because this is local device
 	dev1ReadLimit := rand.Int() % 100000
@@ -87,7 +106,9 @@ func TestSetDeviceLimits(t *testing.T) {
 	dev3ReadLimit := rand.Int() % 10000
 	dev3Conf.MaxRecvKbps = dev3ReadLimit
 
-	waiter, _ := cfg.SetDevices([]config.DeviceConfiguration{dev1Conf, dev2Conf, dev3Conf, dev4Conf})
+	waiter, _ := wrapper.Modify(func(cfg *config.Configuration) {
+		cfg.SetDevices([]config.DeviceConfiguration{dev1Conf, dev2Conf, dev3Conf, dev4Conf})
+	})
 	waiter.Wait()
 
 	expectedR := map[protocol.DeviceID]*rate.Limiter{
@@ -108,10 +129,11 @@ func TestSetDeviceLimits(t *testing.T) {
 }
 
 func TestRemoveDevice(t *testing.T) {
-	cfg := initConfig()
-	lim := newLimiter(device1, cfg)
+	wrapper, wrapperCancel := initConfig()
+	defer wrapperCancel()
+	lim := newLimiter(device1, wrapper)
 
-	waiter, _ := cfg.RemoveDevice(device3)
+	waiter, _ := wrapper.RemoveDevice(device3)
 	waiter.Wait()
 	expectedR := map[protocol.DeviceID]*rate.Limiter{
 		device2: rate.NewLimiter(rate.Limit(dev2Conf.MaxRecvKbps*1024), limiterBurstSize),
@@ -128,15 +150,18 @@ func TestRemoveDevice(t *testing.T) {
 }
 
 func TestAddDevice(t *testing.T) {
-	cfg := initConfig()
-	lim := newLimiter(device1, cfg)
+	wrapper, wrapperCancel := initConfig()
+	defer wrapperCancel()
+	lim := newLimiter(device1, wrapper)
 
 	addedDevice, _ := protocol.DeviceIDFromString("XZJ4UNS-ENI7QGJ-J45DT6G-QSGML2K-6I4XVOG-NAZ7BF5-2VAOWNT-TFDOMQU")
-	addDevConf := config.NewDeviceConfiguration(addedDevice, "addedDevice")
+	addDevConf := newDeviceConfiguration(wrapper, addedDevice, "addedDevice")
 	addDevConf.MaxRecvKbps = 120
 	addDevConf.MaxSendKbps = 240
 
-	waiter, _ := cfg.SetDevice(addDevConf)
+	waiter, _ := wrapper.Modify(func(cfg *config.Configuration) {
+		cfg.SetDevice(addDevConf)
+	})
 	waiter.Wait()
 
 	expectedR := map[protocol.DeviceID]*rate.Limiter{
@@ -159,17 +184,20 @@ func TestAddDevice(t *testing.T) {
 }
 
 func TestAddAndRemove(t *testing.T) {
-	cfg := initConfig()
-	lim := newLimiter(device1, cfg)
+	wrapper, wrapperCancel := initConfig()
+	defer wrapperCancel()
+	lim := newLimiter(device1, wrapper)
 
 	addedDevice, _ := protocol.DeviceIDFromString("XZJ4UNS-ENI7QGJ-J45DT6G-QSGML2K-6I4XVOG-NAZ7BF5-2VAOWNT-TFDOMQU")
-	addDevConf := config.NewDeviceConfiguration(addedDevice, "addedDevice")
+	addDevConf := newDeviceConfiguration(wrapper, addedDevice, "addedDevice")
 	addDevConf.MaxRecvKbps = 120
 	addDevConf.MaxSendKbps = 240
 
-	waiter, _ := cfg.SetDevice(addDevConf)
+	waiter, _ := wrapper.Modify(func(cfg *config.Configuration) {
+		cfg.SetDevice(addDevConf)
+	})
 	waiter.Wait()
-	waiter, _ = cfg.RemoveDevice(device3)
+	waiter, _ = wrapper.RemoveDevice(device3)
 	waiter.Wait()
 
 	expectedR := map[protocol.DeviceID]*rate.Limiter{
@@ -194,7 +222,7 @@ func TestLimitedWriterWrite(t *testing.T) {
 
 	// A buffer with random data that is larger than the write size and not
 	// a precise multiple either.
-	src := make([]byte, int(12.5*maxSingleWriteSize))
+	src := make([]byte, int(12.5*8192))
 	if _, err := crand.Reader.Read(src); err != nil {
 		t.Fatal(err)
 	}
@@ -210,7 +238,7 @@ func TestLimitedWriterWrite(t *testing.T) {
 		writer: cw,
 		waiterHolder: waiterHolder{
 			waiter:    rate.NewLimiter(rate.Limit(42), limiterBurstSize),
-			limitsLAN: new(atomicBool),
+			limitsLAN: new(atomic.Bool),
 			isLAN:     false, // enables limiting
 		},
 	}
@@ -218,9 +246,14 @@ func TestLimitedWriterWrite(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	// Verify there were lots of writes and that the end result is identical.
-	if cw.writeCount != 13 {
-		t.Error("expected lots of smaller writes, but not too many")
+	// Verify there were lots of writes (we expect one kilobyte write size
+	// for the very low rate in this test) and that the end result is
+	// identical.
+	if cw.writeCount < 10*8 {
+		t.Error("expected lots of smaller writes")
+	}
+	if cw.writeCount > 15*8 {
+		t.Error("expected fewer larger writes")
 	}
 	if !bytes.Equal(src, dst.Bytes()) {
 		t.Error("results should be equal")
@@ -234,7 +267,7 @@ func TestLimitedWriterWrite(t *testing.T) {
 		writer: cw,
 		waiterHolder: waiterHolder{
 			waiter:    rate.NewLimiter(rate.Limit(42), limiterBurstSize),
-			limitsLAN: new(atomicBool),
+			limitsLAN: new(atomic.Bool),
 			isLAN:     true, // disables limiting
 		},
 	}
@@ -258,7 +291,7 @@ func TestLimitedWriterWrite(t *testing.T) {
 		writer: cw,
 		waiterHolder: waiterHolder{
 			waiter:    totalWaiter{rate.NewLimiter(rate.Inf, limiterBurstSize), rate.NewLimiter(rate.Inf, limiterBurstSize)},
-			limitsLAN: new(atomicBool),
+			limitsLAN: new(atomic.Bool),
 			isLAN:     false, // enables limiting
 		},
 	}
@@ -286,7 +319,7 @@ func TestLimitedWriterWrite(t *testing.T) {
 				rate.NewLimiter(rate.Limit(42), limiterBurstSize),
 				rate.NewLimiter(rate.Inf, limiterBurstSize),
 			},
-			limitsLAN: new(atomicBool),
+			limitsLAN: new(atomic.Bool),
 			isLAN:     false, // enables limiting
 		},
 	}
@@ -295,8 +328,11 @@ func TestLimitedWriterWrite(t *testing.T) {
 	}
 
 	// Verify there were lots of writes and that the end result is identical.
-	if cw.writeCount != 13 {
-		t.Error("expected just the one write")
+	if cw.writeCount < 10*8 {
+		t.Error("expected lots of smaller writes")
+	}
+	if cw.writeCount > 15*8 {
+		t.Error("expected fewer larger writes")
 	}
 	if !bytes.Equal(src, dst.Bytes()) {
 		t.Error("results should be equal")
