@@ -11,10 +11,15 @@ import (
 	"errors"
 	"fmt"
 
+	"google.golang.org/protobuf/proto"
+
+	"github.com/syncthing/syncthing/internal/gen/bep"
+	"github.com/syncthing/syncthing/internal/gen/dbproto"
 	"github.com/syncthing/syncthing/lib/db/backend"
 	"github.com/syncthing/syncthing/lib/events"
 	"github.com/syncthing/syncthing/lib/osutil"
 	"github.com/syncthing/syncthing/lib/protocol"
+	"github.com/syncthing/syncthing/lib/sliceutil"
 )
 
 var (
@@ -63,48 +68,49 @@ func (t readOnlyTransaction) getFileByKey(key []byte) (protocol.FileInfo, bool,
 	if err != nil || !ok {
 		return protocol.FileInfo{}, false, err
 	}
-	return f.(protocol.FileInfo), true, nil
+	return f, true, nil
 }
 
-func (t readOnlyTransaction) getFileTrunc(key []byte, trunc bool) (protocol.FileIntf, bool, error) {
+func (t readOnlyTransaction) getFileTrunc(key []byte, trunc bool) (protocol.FileInfo, bool, error) {
 	bs, err := t.Get(key)
 	if backend.IsNotFound(err) {
-		return nil, false, nil
+		return protocol.FileInfo{}, false, nil
 	}
 	if err != nil {
-		return nil, false, err
+		return protocol.FileInfo{}, false, err
 	}
 	f, err := t.unmarshalTrunc(bs, trunc)
 	if backend.IsNotFound(err) {
-		return nil, false, nil
+		return protocol.FileInfo{}, false, nil
 	}
 	if err != nil {
-		return nil, false, err
+		return protocol.FileInfo{}, false, err
 	}
 	return f, true, nil
 }
 
-func (t readOnlyTransaction) unmarshalTrunc(bs []byte, trunc bool) (protocol.FileIntf, error) {
+func (t readOnlyTransaction) unmarshalTrunc(bs []byte, trunc bool) (protocol.FileInfo, error) {
 	if trunc {
-		var tf FileInfoTruncated
-		err := tf.Unmarshal(bs)
+		var bfi dbproto.FileInfoTruncated
+		err := proto.Unmarshal(bs, &bfi)
 		if err != nil {
-			return nil, err
+			return protocol.FileInfo{}, err
 		}
-		if err := t.fillTruncated(&tf); err != nil {
-			return nil, err
+		if err := t.fillTruncated(&bfi); err != nil {
+			return protocol.FileInfo{}, err
 		}
-		return tf, nil
+		return protocol.FileInfoFromDBTruncated(&bfi), nil
 	}
 
-	var fi protocol.FileInfo
-	if err := fi.Unmarshal(bs); err != nil {
-		return nil, err
+	var bfi bep.FileInfo
+	err := proto.Unmarshal(bs, &bfi)
+	if err != nil {
+		return protocol.FileInfo{}, err
 	}
-	if err := t.fillFileInfo(&fi); err != nil {
-		return nil, err
+	if err := t.fillFileInfo(&bfi); err != nil {
+		return protocol.FileInfo{}, err
 	}
-	return fi, nil
+	return protocol.FileInfoFromDB(&bfi), nil
 }
 
 type blocksIndirectionError struct {
@@ -121,7 +127,7 @@ func (e *blocksIndirectionError) Unwrap() error {
 
 // fillFileInfo follows the (possible) indirection of blocks and version
 // vector and fills it out.
-func (t readOnlyTransaction) fillFileInfo(fi *protocol.FileInfo) error {
+func (t readOnlyTransaction) fillFileInfo(fi *bep.FileInfo) error {
 	var key []byte
 
 	if len(fi.Blocks) == 0 && len(fi.BlocksHash) != 0 {
@@ -131,8 +137,8 @@ func (t readOnlyTransaction) fillFileInfo(fi *protocol.FileInfo) error {
 		if err != nil {
 			return &blocksIndirectionError{err}
 		}
-		var bl BlockList
-		if err := bl.Unmarshal(bs); err != nil {
+		var bl dbproto.BlockList
+		if err := proto.Unmarshal(bs, &bl); err != nil {
 			return err
 		}
 		fi.Blocks = bl.Blocks
@@ -144,11 +150,11 @@ func (t readOnlyTransaction) fillFileInfo(fi *protocol.FileInfo) error {
 		if err != nil {
 			return fmt.Errorf("filling Version: %w", err)
 		}
-		var v protocol.Vector
-		if err := v.Unmarshal(bs); err != nil {
+		var v bep.Vector
+		if err := proto.Unmarshal(bs, &v); err != nil {
 			return err
 		}
-		fi.Version = v
+		fi.Version = &v
 	}
 
 	return nil
@@ -156,7 +162,7 @@ func (t readOnlyTransaction) fillFileInfo(fi *protocol.FileInfo) error {
 
 // fillTruncated follows the (possible) indirection of version vector and
 // fills it.
-func (t readOnlyTransaction) fillTruncated(fi *FileInfoTruncated) error {
+func (t readOnlyTransaction) fillTruncated(fi *dbproto.FileInfoTruncated) error {
 	var key []byte
 
 	if len(fi.VersionHash) == 0 {
@@ -168,73 +174,72 @@ func (t readOnlyTransaction) fillTruncated(fi *FileInfoTruncated) error {
 	if err != nil {
 		return err
 	}
-	var v protocol.Vector
-	if err := v.Unmarshal(bs); err != nil {
+	var v bep.Vector
+	if err := proto.Unmarshal(bs, &v); err != nil {
 		return err
 	}
-	fi.Version = v
+	fi.Version = &v
 	return nil
 }
 
-func (t readOnlyTransaction) getGlobalVersions(keyBuf, folder, file []byte) (VersionList, error) {
+func (t readOnlyTransaction) getGlobalVersions(keyBuf, folder, file []byte) (*dbproto.VersionList, error) {
 	var err error
 	keyBuf, err = t.keyer.GenerateGlobalVersionKey(keyBuf, folder, file)
 	if err != nil {
-		return VersionList{}, err
+		return nil, err
 	}
 	return t.getGlobalVersionsByKey(keyBuf)
 }
 
-func (t readOnlyTransaction) getGlobalVersionsByKey(key []byte) (VersionList, error) {
+func (t readOnlyTransaction) getGlobalVersionsByKey(key []byte) (*dbproto.VersionList, error) {
 	bs, err := t.Get(key)
 	if err != nil {
-		return VersionList{}, err
+		return nil, err
 	}
 
-	var vl VersionList
-	if err := vl.Unmarshal(bs); err != nil {
-		return VersionList{}, err
+	var vl dbproto.VersionList
+	if err := proto.Unmarshal(bs, &vl); err != nil {
+		return nil, err
 	}
 
-	return vl, nil
+	return &vl, nil
 }
 
-func (t readOnlyTransaction) getGlobal(keyBuf, folder, file []byte, truncate bool) ([]byte, protocol.FileIntf, bool, error) {
+func (t readOnlyTransaction) getGlobal(keyBuf, folder, file []byte, truncate bool) ([]byte, protocol.FileInfo, bool, error) {
 	vl, err := t.getGlobalVersions(keyBuf, folder, file)
 	if backend.IsNotFound(err) {
-		return keyBuf, nil, false, nil
+		return keyBuf, protocol.FileInfo{}, false, nil
 	} else if err != nil {
-		return nil, nil, false, err
+		return nil, protocol.FileInfo{}, false, err
 	}
-	var fi protocol.FileIntf
-	keyBuf, fi, err = t.getGlobalFromVersionList(keyBuf, folder, file, truncate, vl)
+	keyBuf, fi, err := t.getGlobalFromVersionList(keyBuf, folder, file, truncate, vl)
 	return keyBuf, fi, true, err
 }
 
-func (t readOnlyTransaction) getGlobalFromVersionList(keyBuf, folder, file []byte, truncate bool, vl VersionList) ([]byte, protocol.FileIntf, error) {
-	fv, ok := vl.GetGlobal()
+func (t readOnlyTransaction) getGlobalFromVersionList(keyBuf, folder, file []byte, truncate bool, vl *dbproto.VersionList) ([]byte, protocol.FileInfo, error) {
+	fv, ok := vlGetGlobal(vl)
 	if !ok {
-		return keyBuf, nil, errEmptyGlobal
+		return keyBuf, protocol.FileInfo{}, errEmptyGlobal
 	}
 	keyBuf, fi, err := t.getGlobalFromFileVersion(keyBuf, folder, file, truncate, fv)
 	return keyBuf, fi, err
 }
 
-func (t readOnlyTransaction) getGlobalFromFileVersion(keyBuf, folder, file []byte, truncate bool, fv FileVersion) ([]byte, protocol.FileIntf, error) {
-	dev, ok := fv.FirstDevice()
+func (t readOnlyTransaction) getGlobalFromFileVersion(keyBuf, folder, file []byte, truncate bool, fv *dbproto.FileVersion) ([]byte, protocol.FileInfo, error) {
+	dev, ok := fvFirstDevice(fv)
 	if !ok {
-		return keyBuf, nil, errEmptyFileVersion
+		return keyBuf, protocol.FileInfo{}, errEmptyFileVersion
 	}
 	keyBuf, err := t.keyer.GenerateDeviceFileKey(keyBuf, folder, dev, file)
 	if err != nil {
-		return keyBuf, nil, err
+		return keyBuf, protocol.FileInfo{}, err
 	}
 	fi, ok, err := t.getFileTrunc(keyBuf, truncate)
 	if err != nil {
-		return keyBuf, nil, err
+		return keyBuf, protocol.FileInfo{}, err
 	}
 	if !ok {
-		return keyBuf, nil, errEntryFromGlobalMissing
+		return keyBuf, protocol.FileInfo{}, errEntryFromGlobalMissing
 	}
 	return keyBuf, fi, nil
 }
@@ -357,13 +362,13 @@ func (t *readOnlyTransaction) withGlobal(folder, prefix []byte, truncate bool, f
 			return nil
 		}
 
-		var vl VersionList
-		if err := vl.Unmarshal(dbi.Value()); err != nil {
+		var vl dbproto.VersionList
+		if err := proto.Unmarshal(dbi.Value(), &vl); err != nil {
 			return err
 		}
 
-		var f protocol.FileIntf
-		dk, f, err = t.getGlobalFromVersionList(dk, folder, name, truncate, vl)
+		var f protocol.FileInfo
+		dk, f, err = t.getGlobalFromVersionList(dk, folder, name, truncate, &vl)
 		if err != nil {
 			return err
 		}
@@ -432,7 +437,7 @@ func (t *readOnlyTransaction) availability(folder, file []byte) ([]protocol.Devi
 		return nil, err
 	}
 
-	fv, ok := vl.GetGlobal()
+	fv, ok := vlGetGlobal(vl)
 	if !ok {
 		return nil, nil
 	}
@@ -472,32 +477,32 @@ func (t *readOnlyTransaction) withNeedIteratingGlobal(folder, device []byte, tru
 		return err
 	}
 	for dbi.Next() {
-		var vl VersionList
-		if err := vl.Unmarshal(dbi.Value()); err != nil {
+		var vl dbproto.VersionList
+		if err := proto.Unmarshal(dbi.Value(), &vl); err != nil {
 			return err
 		}
 
-		globalFV, ok := vl.GetGlobal()
+		globalFV, ok := vlGetGlobal(&vl)
 		if !ok {
 			return errEmptyGlobal
 		}
-		haveFV, have := vl.Get(device)
+		haveFV, have := vlGet(&vl, device)
 
-		if !Need(globalFV, have, haveFV.Version) {
+		if !Need(globalFV, have, protocol.VectorFromWire(haveFV.Version)) {
 			continue
 		}
 
 		name := t.keyer.NameFromGlobalVersionKey(dbi.Key())
-		var gf protocol.FileIntf
+		var gf protocol.FileInfo
 		dk, gf, err = t.getGlobalFromFileVersion(dk, folder, name, truncate, globalFV)
 		if err != nil {
 			return err
 		}
 
 		if shouldDebug() {
-			if globalDev, ok := globalFV.FirstDevice(); ok {
+			if globalDev, ok := fvFirstDevice(globalFV); ok {
 				globalID, _ := protocol.DeviceIDFromBytes(globalDev)
-				l.Debugf("need folder=%q device=%v name=%q have=%v invalid=%v haveV=%v haveDeleted=%v globalV=%v globalDeleted=%v globalDev=%v", folder, devID, name, have, haveFV.IsInvalid(), haveFV.Version, haveFV.Deleted, gf.FileVersion(), globalFV.Deleted, globalID)
+				l.Debugf("need folder=%q device=%v name=%q have=%v invalid=%v haveV=%v haveDeleted=%v globalV=%v globalDeleted=%v globalDev=%v", folder, devID, name, have, fvIsInvalid(haveFV), haveFV.Version, haveFV.Deleted, gf.FileVersion(), globalFV.Deleted, globalID)
 			}
 		}
 		if !fn(gf) {
@@ -519,7 +524,7 @@ func (t *readOnlyTransaction) withNeedLocal(folder []byte, truncate bool, fn Ite
 	defer dbi.Release()
 
 	var keyBuf []byte
-	var f protocol.FileIntf
+	var f protocol.FileInfo
 	var ok bool
 	for dbi.Next() {
 		keyBuf, f, ok, err = t.getGlobal(keyBuf, folder, t.keyer.NameFromGlobalVersionKey(dbi.Key()), truncate)
@@ -589,7 +594,8 @@ func (t readWriteTransaction) putFile(fkey []byte, fi protocol.FileInfo) error {
 		bkey = t.keyer.GenerateBlockListKey(bkey, fi.BlocksHash)
 		if _, err := t.Get(bkey); backend.IsNotFound(err) {
 			// Marshal the block list and save it
-			blocksBs := mustMarshal(&BlockList{Blocks: fi.Blocks})
+			blocks := sliceutil.Map(fi.Blocks, protocol.BlockInfo.ToWire)
+			blocksBs := mustMarshal(&dbproto.BlockList{Blocks: blocks})
 			if err := t.Put(bkey, blocksBs); err != nil {
 				return err
 			}
@@ -605,7 +611,7 @@ func (t readWriteTransaction) putFile(fkey []byte, fi protocol.FileInfo) error {
 		bkey = t.keyer.GenerateVersionKey(bkey, fi.VersionHash)
 		if _, err := t.Get(bkey); backend.IsNotFound(err) {
 			// Marshal the version vector and save it
-			versionBs := mustMarshal(&fi.Version)
+			versionBs := mustMarshal(fi.Version.ToWire())
 			if err := t.Put(bkey, versionBs); err != nil {
 				return err
 			}
@@ -619,7 +625,7 @@ func (t readWriteTransaction) putFile(fkey []byte, fi protocol.FileInfo) error {
 
 	t.indirectionTracker.recordIndirectionHashesForFile(&fi)
 
-	fiBs := mustMarshal(&fi)
+	fiBs := mustMarshal(fi.ToWire(true))
 	return t.Put(fkey, fiBs)
 }
 
@@ -638,8 +644,11 @@ func (t readWriteTransaction) updateGlobal(gk, keyBuf, folder, device []byte, fi
 	if err != nil && !backend.IsNotFound(err) {
 		return nil, err
 	}
+	if fl == nil {
+		fl = &dbproto.VersionList{}
+	}
 
-	globalFV, oldGlobalFV, removedFV, haveOldGlobal, haveRemoved, globalChanged, err := fl.update(folder, device, file, t.readOnlyTransaction)
+	globalFV, oldGlobalFV, removedFV, haveOldGlobal, haveRemoved, globalChanged, err := vlUpdate(fl, folder, device, file, t.readOnlyTransaction)
 	if err != nil {
 		return nil, err
 	}
@@ -647,20 +656,20 @@ func (t readWriteTransaction) updateGlobal(gk, keyBuf, folder, device []byte, fi
 	name := []byte(file.Name)
 
 	l.Debugf(`new global for "%v" after update: %v`, file.Name, fl)
-	if err := t.Put(gk, mustMarshal(&fl)); err != nil {
+	if err := t.Put(gk, mustMarshal(fl)); err != nil {
 		return nil, err
 	}
 
 	// Only load those from db if actually needed
 
 	var gotGlobal, gotOldGlobal bool
-	var global, oldGlobal protocol.FileIntf
+	var global, oldGlobal protocol.FileInfo
 
 	// Check the need of the device that was updated
 	// Must happen before updating global meta: If this is the first
 	// item from this device, it will be initialized with the global state.
 
-	needBefore := haveOldGlobal && Need(oldGlobalFV, haveRemoved, removedFV.Version)
+	needBefore := haveOldGlobal && Need(oldGlobalFV, haveRemoved, protocol.VectorFromWire(removedFV.GetVersion()))
 	needNow := Need(globalFV, true, file.Version)
 	if needBefore {
 		if keyBuf, oldGlobal, err = t.getGlobalFromFileVersion(keyBuf, folder, name, true, oldGlobalFV); err != nil {
@@ -709,7 +718,7 @@ func (t readWriteTransaction) updateGlobal(gk, keyBuf, folder, device []byte, fi
 
 	// Add the new global to the global size counter
 	if !gotGlobal {
-		if globalFV.Version.Equal(file.Version) {
+		if protocol.VectorFromWire(globalFV.Version).Equal(file.Version) {
 			// The inserted file is the global file
 			global = file
 		} else {
@@ -718,15 +727,15 @@ func (t readWriteTransaction) updateGlobal(gk, keyBuf, folder, device []byte, fi
 				return nil, err
 			}
 		}
-		gotGlobal = true
 	}
 	meta.addFile(protocol.GlobalDeviceID, global)
 
 	// check for local (if not already done before)
 	if !bytes.Equal(device, protocol.LocalDeviceID[:]) {
-		localFV, haveLocal := fl.Get(protocol.LocalDeviceID[:])
-		needBefore := haveOldGlobal && Need(oldGlobalFV, haveLocal, localFV.Version)
-		needNow := Need(globalFV, haveLocal, localFV.Version)
+		localFV, haveLocal := vlGet(fl, protocol.LocalDeviceID[:])
+		localVersion := protocol.VectorFromWire(localFV.Version)
+		needBefore := haveOldGlobal && Need(oldGlobalFV, haveLocal, localVersion)
+		needNow := Need(globalFV, haveLocal, localVersion)
 		if needBefore {
 			meta.removeNeeded(protocol.LocalDeviceID, oldGlobal)
 			if !needNow {
@@ -750,11 +759,12 @@ func (t readWriteTransaction) updateGlobal(gk, keyBuf, folder, device []byte, fi
 			// Already handled above
 			continue
 		}
-		fv, have := fl.Get(dev[:])
-		if haveOldGlobal && Need(oldGlobalFV, have, fv.Version) {
+		fv, have := vlGet(fl, dev[:])
+		fvVersion := protocol.VectorFromWire(fv.Version)
+		if haveOldGlobal && Need(oldGlobalFV, have, fvVersion) {
 			meta.removeNeeded(dev, oldGlobal)
 		}
-		if Need(globalFV, have, fv.Version) {
+		if Need(globalFV, have, fvVersion) {
 			meta.addNeeded(dev, global)
 		}
 	}
@@ -778,11 +788,12 @@ func (t readWriteTransaction) updateLocalNeed(keyBuf, folder, name []byte, add b
 	return keyBuf, err
 }
 
-func Need(global FileVersion, haveLocal bool, localVersion protocol.Vector) bool {
+func Need(global *dbproto.FileVersion, haveLocal bool, localVersion protocol.Vector) bool {
 	// We never need an invalid file or a file without a valid version (just
 	// another way of expressing "invalid", really, until we fix that
 	// part...).
-	if global.IsInvalid() || global.Version.IsEmpty() {
+	globalVersion := protocol.VectorFromWire(global.Version)
+	if fvIsInvalid(global) || globalVersion.IsEmpty() {
 		return false
 	}
 	// We don't need a deleted file if we don't have it.
@@ -790,7 +801,7 @@ func Need(global FileVersion, haveLocal bool, localVersion protocol.Vector) bool
 		return false
 	}
 	// We don't need the global file if we already have the same version.
-	if haveLocal && localVersion.GreaterEqual(global.Version) {
+	if haveLocal && localVersion.GreaterEqual(globalVersion) {
 		return false
 	}
 	return true
@@ -816,8 +827,8 @@ func (t readWriteTransaction) removeFromGlobal(gk, keyBuf, folder, device, file
 		return nil, err
 	}
 
-	oldGlobalFV, haveOldGlobal := fl.GetGlobal()
-	oldGlobalFV = oldGlobalFV.copy()
+	oldGlobalFV, haveOldGlobal := vlGetGlobal(fl)
+	oldGlobalFV = fvCopy(oldGlobalFV)
 
 	if !haveOldGlobal {
 		// Shouldn't ever happen, but doesn't hurt to handle.
@@ -825,21 +836,18 @@ func (t readWriteTransaction) removeFromGlobal(gk, keyBuf, folder, device, file
 		return keyBuf, t.Delete(gk)
 	}
 
-	removedFV, haveRemoved, globalChanged, err := fl.pop(device, file)
-	if err != nil {
-		return nil, err
-	}
+	removedFV, haveRemoved, globalChanged := vlPop(fl, device)
 	if !haveRemoved {
 		// There is no version for the given device
 		return keyBuf, nil
 	}
 
-	var global protocol.FileIntf
+	var global protocol.FileInfo
 	var gotGlobal bool
 
-	globalFV, haveGlobal := fl.GetGlobal()
+	globalFV, haveGlobal := vlGetGlobal(fl)
 	// Add potential needs of the removed device
-	if haveGlobal && !globalFV.IsInvalid() && Need(globalFV, false, protocol.Vector{}) && !Need(oldGlobalFV, haveRemoved, removedFV.Version) {
+	if haveGlobal && !fvIsInvalid(globalFV) && Need(globalFV, false, protocol.Vector{}) && !Need(oldGlobalFV, haveRemoved, protocol.VectorFromWire(removedFV.Version)) {
 		keyBuf, global, err = t.getGlobalFromVersionList(keyBuf, folder, file, true, fl)
 		if err != nil {
 			return nil, err
@@ -856,13 +864,13 @@ func (t readWriteTransaction) removeFromGlobal(gk, keyBuf, folder, device, file
 	// Global hasn't changed, abort early
 	if !globalChanged {
 		l.Debugf("new global after remove: %v", fl)
-		if err := t.Put(gk, mustMarshal(&fl)); err != nil {
+		if err := t.Put(gk, mustMarshal(fl)); err != nil {
 			return nil, err
 		}
 		return keyBuf, nil
 	}
 
-	var oldGlobal protocol.FileIntf
+	var oldGlobal protocol.FileInfo
 	keyBuf, oldGlobal, err = t.getGlobalFromFileVersion(keyBuf, folder, file, true, oldGlobalFV)
 	if err != nil {
 		return nil, err
@@ -871,11 +879,12 @@ func (t readWriteTransaction) removeFromGlobal(gk, keyBuf, folder, device, file
 
 	// Remove potential device needs
 	shouldRemoveNeed := func(dev protocol.DeviceID) bool {
-		fv, have := fl.Get(dev[:])
-		if !Need(oldGlobalFV, have, fv.Version) {
+		fv, have := vlGet(fl, dev[:])
+		fvVersion := protocol.VectorFromWire(fv.Version)
+		if !Need(oldGlobalFV, have, fvVersion) {
 			return false // Didn't need it before
 		}
-		return !haveGlobal || !Need(globalFV, have, fv.Version)
+		return !haveGlobal || !Need(globalFV, have, fvVersion)
 	}
 	if shouldRemoveNeed(protocol.LocalDeviceID) {
 		meta.removeNeeded(protocol.LocalDeviceID, oldGlobal)
@@ -893,7 +902,7 @@ func (t readWriteTransaction) removeFromGlobal(gk, keyBuf, folder, device, file
 	}
 
 	// Nothing left, i.e. nothing to add to the global counter below.
-	if fl.Empty() {
+	if len(fl.Versions) == 0 {
 		if err := t.Delete(gk); err != nil {
 			return nil, err
 		}
@@ -910,7 +919,7 @@ func (t readWriteTransaction) removeFromGlobal(gk, keyBuf, folder, device, file
 	meta.addFile(protocol.GlobalDeviceID, global)
 
 	l.Debugf(`new global for "%s" after remove: %v`, file, fl)
-	if err := t.Put(gk, mustMarshal(&fl)); err != nil {
+	if err := t.Put(gk, mustMarshal(fl)); err != nil {
 		return nil, err
 	}
 
@@ -938,7 +947,7 @@ func (t readWriteTransaction) deleteKeyPrefixMatching(prefix []byte, match func(
 	return dbi.Error()
 }
 
-func (t *readWriteTransaction) withAllFolderTruncated(folder []byte, fn func(device []byte, f FileInfoTruncated) bool) error {
+func (t *readWriteTransaction) withAllFolderTruncated(folder []byte, fn func(device []byte, f protocol.FileInfo) bool) error {
 	key, err := t.keyer.GenerateDeviceFileKey(nil, folder, nil, nil)
 	if err != nil {
 		return err
@@ -960,11 +969,10 @@ func (t *readWriteTransaction) withAllFolderTruncated(folder []byte, fn func(dev
 			continue
 		}
 
-		intf, err := t.unmarshalTrunc(dbi.Value(), true)
+		f, err := t.unmarshalTrunc(dbi.Value(), true)
 		if err != nil {
 			return err
 		}
-		f := intf.(FileInfoTruncated)
 
 		switch f.Name {
 		case "", ".", "..", "/": // A few obviously invalid filenames
@@ -991,12 +999,8 @@ func (t *readWriteTransaction) withAllFolderTruncated(folder []byte, fn func(dev
 	return dbi.Error()
 }
 
-type marshaller interface {
-	Marshal() ([]byte, error)
-}
-
-func mustMarshal(f marshaller) []byte {
-	bs, err := f.Marshal()
+func mustMarshal(f proto.Message) []byte {
+	bs, err := proto.Marshal(f)
 	if err != nil {
 		panic(err)
 	}
