@@ -7,16 +7,7 @@
 package events
 
 import (
-	"os"
-	"strings"
-
-	"github.com/syncthing/syncthing/lib/logger"
+	liblogger "github.com/syncthing/syncthing/lib/logger"
 )
 
-var (
-	dl = logger.DefaultLogger.NewFacility("events", "Event generation and logging")
-)
-
-func init() {
-	dl.SetDebug("events", strings.Contains(os.Getenv("STTRACE"), "events") || os.Getenv("STTRACE") == "all")
-}
+var dl = liblogger.DefaultLogger.NewFacility("events", "Event generation and logging")
