@@ -9,7 +9,6 @@ package fs
 import (
 	"errors"
 	"fmt"
-	"os"
 	"path/filepath"
 	"runtime"
 	"sort"
@@ -28,17 +27,20 @@ func TestRealCase(t *testing.T) {
 		testRealCase(t, newFakeFilesystem(t.Name()+"?insens=true"))
 	})
 	t.Run("actual", func(t *testing.T) {
-		fsys, tmpDir := setup(t)
-		defer os.RemoveAll(tmpDir)
+		fsys, _ := setup(t)
 		testRealCase(t, fsys)
 	})
 }
 
+func newCaseFilesystem(fsys Filesystem) *caseFilesystem {
+	return globalCaseFilesystemRegistry.get(fsys).(*caseFilesystem)
+}
+
 func testRealCase(t *testing.T, fsys Filesystem) {
-	testFs := NewCaseFilesystem(fsys).(*caseFilesystem)
+	testFs := newCaseFilesystem(fsys)
 	comps := []string{"Foo", "bar", "BAZ", "bAs"}
 	path := filepath.Join(comps...)
-	testFs.MkdirAll(filepath.Join(comps[:len(comps)-1]...), 0777)
+	testFs.MkdirAll(filepath.Join(comps[:len(comps)-1]...), 0o777)
 	fd, err := testFs.Create(path)
 	if err != nil {
 		t.Fatal(err)
@@ -79,20 +81,19 @@ func TestRealCaseSensitive(t *testing.T) {
 		testRealCaseSensitive(t, newFakeFilesystem(t.Name()))
 	})
 	t.Run("actual", func(t *testing.T) {
-		fsys, tmpDir := setup(t)
-		defer os.RemoveAll(tmpDir)
+		fsys, _ := setup(t)
 		testRealCaseSensitive(t, fsys)
 	})
 }
 
 func testRealCaseSensitive(t *testing.T, fsys Filesystem) {
-	testFs := NewCaseFilesystem(fsys).(*caseFilesystem)
+	testFs := newCaseFilesystem(fsys)
 
 	names := make([]string, 2)
 	names[0] = "foo"
 	names[1] = strings.ToUpper(names[0])
 	for _, n := range names {
-		if err := testFs.MkdirAll(n, 0777); err != nil {
+		if err := testFs.MkdirAll(n, 0o777); err != nil {
 			if IsErrCaseConflict(err) {
 				t.Skip("Filesystem is case-insensitive")
 			}
@@ -120,8 +121,7 @@ func TestCaseFSStat(t *testing.T) {
 		testCaseFSStat(t, newFakeFilesystem(t.Name()+"?insens=true"))
 	})
 	t.Run("actual", func(t *testing.T) {
-		fsys, tmpDir := setup(t)
-		defer os.RemoveAll(tmpDir)
+		fsys, _ := setup(t)
 		testCaseFSStat(t, fsys)
 	})
 }
@@ -139,7 +139,7 @@ func testCaseFSStat(t *testing.T, fsys Filesystem) {
 		sensitive = false
 	}
 
-	testFs := NewCaseFilesystem(fsys)
+	testFs := newCaseFilesystem(fsys)
 	_, err = testFs.Stat("FOO")
 	if sensitive {
 		if IsNotExist(err) {
@@ -175,7 +175,10 @@ func BenchmarkWalkCaseFakeFS100k(b *testing.B) {
 		// Construct the casefs manually or it will get cached and the benchmark is invalid.
 		casefs := &caseFilesystem{
 			Filesystem: fsys,
-			realCaser:  newDefaultRealCaser(fsys),
+			realCaser: &defaultRealCaser{
+				fs:    fsys,
+				cache: newCaseCache(),
+			},
 		}
 		var fakefs *fakeFS
 		if ffs, ok := unwrapFilesystem(fsys, filesystemWrapperTypeNone); ok {
@@ -201,7 +204,10 @@ func BenchmarkWalkCaseFakeFS100k(b *testing.B) {
 		// Construct the casefs manually or it will get cached and the benchmark is invalid.
 		casefs := &caseFilesystem{
 			Filesystem: fsys,
-			realCaser:  newDefaultRealCaser(fsys),
+			realCaser: &defaultRealCaser{
+				fs:    fsys,
+				cache: newCaseCache(),
+			},
 		}
 		var fakefs *fakeFS
 		if ffs, ok := unwrapFilesystem(fsys, filesystemWrapperTypeNone); ok {
@@ -242,7 +248,7 @@ func benchmarkWalkFakeFS(b *testing.B, fsys Filesystem, paths []string, otherOpE
 }
 
 func TestStressCaseFS(t *testing.T) {
-	// Exercise a bunch of paralell operations for stressing out race
+	// Exercise a bunch of parallel operations for stressing out race
 	// conditions in the realnamer cache etc.
 
 	const limit = 10 * time.Second
