@@ -1,8 +1,14 @@
-// Copyright (C) 2015 The Protocol Authors.
+// Copyright (C) 2015 The Syncthing Authors.
+//
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this file,
+// You can obtain one at https://mozilla.org/MPL/2.0/.
 
 package protocol
 
-import "testing"
+import (
+	"testing"
+)
 
 func TestWinsConflict(t *testing.T) {
 	testcases := [][2]FileInfo{
