@@ -14,6 +14,8 @@ import (
 	"runtime"
 	"strings"
 	"testing"
+
+	"github.com/syncthing/syncthing/lib/build"
 )
 
 var versions = []struct {
@@ -120,7 +122,7 @@ func TestSelectedRelease(t *testing.T) {
 }
 
 func TestSelectedReleaseMacOS(t *testing.T) {
-	if runtime.GOOS != "darwin" {
+	if !build.IsDarwin {
 		t.Skip("macOS only")
 	}
 
@@ -142,7 +144,7 @@ func TestSelectedReleaseMacOS(t *testing.T) {
 			},
 		}
 
-		// Check that it is selected and the asset is as epected
+		// Check that it is selected and the asset is as expected
 		sel, err := SelectLatestRelease(rels, "v0.14.46", false)
 		if err != nil {
 			t.Fatal("Unexpected error:", err)
