@@ -4,6 +4,7 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this file,
 // You can obtain one at https://mozilla.org/MPL/2.0/.
 
+//go:build integration
 // +build integration
 
 package integration
@@ -202,7 +203,7 @@ func TestSyncCluster(t *testing.T) {
 		if err != nil {
 			t.Fatal(err)
 		}
-		fd, err := os.OpenFile("s1/test-appendfile", os.O_APPEND|os.O_WRONLY, 0644)
+		fd, err := os.OpenFile("s1/test-appendfile", os.O_APPEND|os.O_WRONLY, 0o644)
 		if err != nil {
 			t.Fatal(err)
 		}
@@ -325,7 +326,7 @@ func TestSyncSparseFile(t *testing.T) {
 
 	log.Println("Generating files...")
 
-	if err := os.Mkdir("s1", 0755); err != nil {
+	if err := os.Mkdir("s1", 0o755); err != nil {
 		t.Fatal(err)
 	}
 
